// src/transform.ts
import { parse } from "@babel/parser";
import { walkAST } from "ast-walker-scope";
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : Array.isArray(test) ? test.includes(node.callee.name) : test(node.callee.name)));
}
function getTransformResult(s, id) {
  if (s?.hasChanged()) {
    return {
      code: s.toString(),
      get map() {
        return s.generateMap({
          source: id,
          includeContent: true,
          hires: true
        });
      }
    };
  }
}
function parseAst(code, lang = "") {
  const plugins = [];
  if (/tsx?$/.test(lang)) {
    plugins.push("typescript");
  }
  if (lang.endsWith("x")) {
    plugins.push("jsx");
  }
  return parse(code, {
    sourceType: "module",
    plugins
  });
}
function transformAst(ast, s, option, offset = 0) {
  walkAST(ast, {
    enter(node) {
      if (isCallOf(node, "dynamicImportComponents")) {
        const [type, ...args] = node.arguments.map((node2) => {
          if (node2.type === "StringLiteral") {
            return node2.value;
          } else {
            throw new SyntaxError("dynamicImportComponents argument must be string literal");
          }
        });
        if (Reflect.has(option, type)) {
          s.overwriteNode(node, option[type](...args), { offset });
        } else {
          throw new Error(`dynamicImportComponents type "${type}" not found`);
        }
      }
    }
  });
}

// src/common.ts
var REGEX_TS_FILE = /\.[cm]?[jt]sx?$/;
var REGEX_SETUP_SFC = /\.setup\.[cm]?[jt]sx?$/;
var REGEX_VUE_SFC = /\.vue$/;

// src/index.ts
import { createFilter } from "@rollup/pluginutils";
import { MagicString } from "magic-string-ast";
import { parse as parseSFC } from "@vue/compiler-sfc";
function CreateDynamicImportComponentsPlugin(option) {
  const filter = createFilter([REGEX_TS_FILE, REGEX_SETUP_SFC, REGEX_VUE_SFC]);
  return {
    name: "sort-dynamic-import",
    enforce: "pre",
    transform(code, id) {
      if (!filter(id))
        return;
      if (!code.includes("dynamicImportComponent"))
        return;
      const s = new MagicString(code);
      if (REGEX_VUE_SFC.test(id)) {
        const { descriptor } = parseSFC(code, { filename: id });
        if (descriptor.scriptSetup) {
          const setUpAst = parseAst(descriptor.scriptSetup.content, descriptor.scriptSetup.lang);
          transformAst(setUpAst, s, option, descriptor.scriptSetup.loc.start.offset);
        }
        if (descriptor.script) {
          const scriptAst = parseAst(descriptor.script.content, descriptor.script.lang);
          transformAst(scriptAst, s, option, descriptor.script.loc.start.offset);
        }
      } else {
        let lang = /\.tsx?$/.test(id) ? "ts" : "js";
        if (id.endsWith("x")) {
          lang += "x";
        }
        const ast = parseAst(code, lang);
        transformAst(ast, s, option);
      }
      return getTransformResult(s, id);
    }
  };
}
export {
  CreateDynamicImportComponentsPlugin as default
};
